import { useState, useEffect, useRef, useCallback } from 'react';
import "./styles.sass";

export const MusicSlider = ({ isReadOnly, onFrequencyChange, frequency: externalFrequency, composerIsPlaying }) => {
  const [audioContext, setAudioContext] = useState(null);
  const [oscillator, setOscillator] = useState(null);
  const [gainNode, setGainNode] = useState(null);
  const sliderRef = useRef(null);
  const frequency = useRef(externalFrequency || 440);
  const isPressed = useRef(false);
  const isDragging = useRef(false);

  const initAudio = useCallback(() => {
    if (audioContext) return;

    try {
      const ctx = new (window.AudioContext || window.webkitAudioContext)();
      const osc = ctx.createOscillator();
      const gain = ctx.createGain();

      osc.type = 'sine';
      osc.frequency.setValueAtTime(frequency.current, ctx.currentTime);

      gain.gain.setValueAtTime(0.00001, ctx.currentTime);

      osc.connect(gain);
      gain.connect(ctx.destination);
      osc.start();

      setAudioContext(ctx);
      setOscillator(osc);
      setGainNode(gain);

      if (isReadOnly) {
        gain.gain.setValueAtTime(0.00001, ctx.currentTime);
      }
    } catch (error) {
      console.error("Audio initialization error:", error);
    }
  }, [audioContext, isReadOnly]);

  useEffect(() => {
    if (!isReadOnly || !externalFrequency) return;
    
    // Initialize audio if not already done
    if (!audioContext) {
      initAudio();
      return;
    }

    if (oscillator && gainNode) {
      frequency.current = externalFrequency;
      oscillator.frequency.setValueAtTime(frequency.current, audioContext.currentTime);

      if (composerIsPlaying) {
        gainNode.gain.setTargetAtTime(0.5, audioContext.currentTime, 0.01);
      } else {
        gainNode.gain.setTargetAtTime(0.00001, audioContext.currentTime, 0.05);
      }
    }
  }, [externalFrequency, composerIsPlaying, oscillator, audioContext, gainNode, isReadOnly, initAudio]);

  useEffect(() => {
    if (isReadOnly) return;

    const calculateFrequency = (clientX) => {
      const slider = sliderRef.current;
      const rect = slider.getBoundingClientRect();
      const width = rect.width;
      
      // Calculate relative X position using window coordinates
      let x = clientX - rect.left;
      
      // For positions outside the slider, calculate relative to slider bounds
      if (clientX < rect.left) x = 0;
      if (clientX > rect.right) x = width;
      
      const percentage = x / width;
      return Math.round(220 + percentage * 880);
    };

    const handleMouseMove = (e) => {
      if (!isPressed.current) return;

      e.preventDefault();
      e.stopPropagation();

      const newFreq = calculateFrequency(e.clientX);
      frequency.current = newFreq;
      onFrequencyChange(newFreq, true);

      if (oscillator && gainNode) {
        oscillator.frequency.setValueAtTime(newFreq, audioContext.currentTime);
        gainNode.gain.setTargetAtTime(0.5, audioContext.currentTime, 0.01);
      }
    };

    const handleMouseDown = (e) => {
      const target = e.target;
      const isSlider = target.classList.contains('otamatone-neck');
      const isIndicator = target.classList.contains('frequency-indicator') || 
                         target.closest('.frequency-indicator');
      
      if (!isSlider && !isIndicator) return;

      e.preventDefault();
      e.stopPropagation();

      if (!audioContext) initAudio();
      isPressed.current = true;
      isDragging.current = true;
      
      if (gainNode) {
        gainNode.gain.setTargetAtTime(0.5, audioContext.currentTime, 0.01);
      }

      document.body.style.userSelect = 'none';
      const iframes = document.querySelectorAll('iframe');
      iframes.forEach(iframe => {
        iframe.style.pointerEvents = 'none';
      });
      
      onFrequencyChange(frequency.current, true);
      handleMouseMove(e);
    };

    const handleMouseUp = (e) => {
      if (!isPressed.current) return;
      
      e.preventDefault();
      e.stopPropagation();
      
      isPressed.current = false;
      isDragging.current = false;
      
      if (gainNode) {
        gainNode.gain.setTargetAtTime(0.00001, audioContext.currentTime, 0.05);
      }

      document.body.style.userSelect = '';
      const iframes = document.querySelectorAll('iframe');
      iframes.forEach(iframe => {
        iframe.style.pointerEvents = 'auto';
      });
      
      onFrequencyChange(frequency.current, false);
    };

    document.addEventListener('mousemove', handleMouseMove, true);
    document.addEventListener('mouseup', handleMouseUp, true);
    
    const slider = sliderRef.current;
    if (slider) {
      slider.addEventListener('mousedown', handleMouseDown, true);
    }

    return () => {
      document.removeEventListener('mousemove', handleMouseMove, true);
      document.removeEventListener('mouseup', handleMouseUp, true);
      if (slider) {
        slider.removeEventListener('mousedown', handleMouseDown, true);
      }
      document.body.style.userSelect = '';
      const iframes = document.querySelectorAll('iframe');
      iframes.forEach(iframe => {
        iframe.style.pointerEvents = 'auto';
      });
    };
  }, [isReadOnly, onFrequencyChange, audioContext, oscillator, gainNode, initAudio]);

  useEffect(() => {
    return () => {
      if (gainNode) gainNode.gain.setValueAtTime(0, audioContext.currentTime);
      if (oscillator) {
        try {
          oscillator.stop();
        } catch (e) {
          console.warn("Oscillator already stopped");
        }
      }
      if (audioContext) audioContext.close();
    };
  }, [oscillator, audioContext, gainNode]);

  const getFrequencyPosition = () => {
    const pos = ((frequency.current - 220) / 880) * 100;
    return `${Math.max(0, Math.min(pos, 100))}%`;
  };

  return (
    <div className="otamatone-container">
      <div className="otamatone">
        <div className="otamatone-face">
          <div className="otamatone-eyes">
            <div className="eye left-eye"></div>
            <div className="eye right-eye"></div>
          </div>
          <div className="otamatone-mouth"></div>
        </div>
        <div 
          ref={sliderRef}
          className={`otamatone-neck ${!isReadOnly ? 'interactive' : ''}`}
          style={{ cursor: isReadOnly ? 'default' : 'pointer' }}
        >
          <div
            className={`frequency-indicator ${isReadOnly ? 'read-only' : ''} ${composerIsPlaying ? 'active' : ''}`}
            style={{ left: getFrequencyPosition() }}
          >
            <div className="note-marker">
              {(!isReadOnly && isDragging.current) && Math.round(frequency.current)}
            </div>
          </div>
        </div>
      </div>
    </div>
  );
};