import { useContext, useEffect, useState, useRef } from "react";
import { SocketContext } from "@/common/contexts/SocketContext";
import { StateContext } from "@/common/contexts/StateContext";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { faPlayCircle, faUsers, faMessage, faArrowLeft, faCopy, faCheck } from "@fortawesome/free-solid-svg-icons";
import "./styles.sass";

export const WaitingRoom = () => {
    const { socket, on, send } = useContext(SocketContext);
    const { setCurrentState } = useContext(StateContext);
    const [users, setUsers] = useState([]);
    const [messages, setMessages] = useState([]);
    const [inputValue, setInputValue] = useState("");
    const [roomCode, setRoomCode] = useState("");
    const [isHost, setIsHost] = useState(false);
    const [username, setUsername] = useState("");
    const [copied, setCopied] = useState(false);
    const [playlists, setPlaylists] = useState({});
    const [votes, setVotes] = useState({});
    const [selectedPlaylist, setSelectedPlaylist] = useState(null);
    const messageEndRef = useRef(null);

    useEffect(() => {
        send("check-host-status");
        send("get-user-info");
        send("get-room-users");
        send("get-room-code");
        send("get-playlist-options");

        const handleHostStatus = (status) => {
            setIsHost(status.isHost);
        };

        const handleUserInfo = (userInfo) => {
            setUsername(userInfo.name);
        };

        const handleRoomUsers = (users) => {
            setUsers(users);
        };

        const handleRoomUsersUpdate = (updatedUsers) => {
            setUsers(updatedUsers);
        };

        const handleRoomCode = (code) => {
            setRoomCode(code);
        };

        const handleUserConnected = (userData) => {
            setMessages(prev => [...prev, {
                system: true,
                text: `${userData.name} ist beigetreten`
            }]);
        };

        const handleUserDisconnected = (userId) => {
            setUsers(prev => {
                const user = prev.find(u => u.id === userId);
                if (user) {
                    setMessages(prevMsgs => [...prevMsgs, {
                        system: true,
                        text: `${user.name} hat den Raum verlassen`
                    }]);
                }
                return prev.filter(u => u.id !== userId);
            });
        };

        const handleChatMessage = (messageData) => {
            setMessages(prev => [...prev, messageData]);
        };

        const handleGameStarted = () => {
            setCurrentState("Game");
        };

        const handlePlaylistOptions = (options) => {
            setPlaylists(options);
        };

        const handleVotesUpdated = (newVotes) => {
            setVotes(newVotes);
        };

        // Register event listeners
        const cleanupHostStatus = on("host-status", handleHostStatus);
        const cleanupUserInfo = on("user-info", handleUserInfo);
        const cleanupRoomUsers = on("room-users", handleRoomUsers);
        const cleanupRoomCode = on("room-code", handleRoomCode);
        const cleanupRoomUsersUpdate = on("room-users-update", handleRoomUsersUpdate);
        const cleanupUserConnected = on("user-connected", handleUserConnected);
        const cleanupUserDisconnected = on("user-disconnected", handleUserDisconnected);
        const cleanupChatMessage = on("chat-message", handleChatMessage);
        const cleanupGameStarted = on("game-started", handleGameStarted);
        const cleanupPlaylistOptions = on("playlist-options", handlePlaylistOptions);
        const cleanupVotesUpdated = on("playlist-votes-updated", handleVotesUpdated);

        setMessages([{
            system: true,
            text: "Welcome to the waiting room! Waiting for others to join..."
        }]);

        return () => {
            cleanupHostStatus();
            cleanupUserInfo();
            cleanupRoomUsers();
            cleanupRoomCode();
            cleanupRoomUsersUpdate();
            cleanupUserConnected();
            cleanupUserDisconnected();
            cleanupChatMessage();
            cleanupGameStarted();
            cleanupPlaylistOptions();
            cleanupVotesUpdated();
        };
    }, [on, send, socket, setCurrentState]);

    useEffect(() => {
        messageEndRef.current?.scrollIntoView({ behavior: "smooth" });
    }, [messages]);

    const handleSendMessage = () => {
        if (inputValue.trim()) {
            const messageData = {
                text: inputValue,
                sender: username
            };
            send("send-message", messageData);
            setMessages(prev => [...prev, messageData]);
            setInputValue("");
        }
    };

    const handleStartGame = () => {
        if (isHost) {
            send("start-game");
        }
    };

    const handleLeaveRoom = () => {
        if (socket) {
            socket.disconnect();
        }
        setCurrentState("Home");
    };

    const copyRoomCode = () => {
        navigator.clipboard.writeText(roomCode);
        setCopied(true);
        setTimeout(() => setCopied(false), 2000);
    };

    const handleVote = (playlistId) => {
        setSelectedPlaylist(playlistId);
        send("vote-playlist", { playlistId });
    };

    const getVoteCount = (playlistId) => {
        return votes[playlistId]?.length || 0;
    };

    const renderPlaylistSection = () => (
        <div className="playlist-section">
            <div className="section-header">
                <h2>Choose a Playlist</h2>
                <p className="vote-info">Most voted playlist will be used for the game</p>
            </div>
            <div className="playlists-grid">
                {Object.entries(playlists).map(([genre, playlist]) => (
                    <div 
                        key={playlist.id}
                        className={`playlist-card ${selectedPlaylist === playlist.id ? 'selected' : ''}`}
                        onClick={() => handleVote(playlist.id)}
                    >
                        <div className="playlist-thumbnail">
                            <img src={playlist.thumbnail.url} alt={playlist.title} />
                            <div className="playlist-overlay">
                                <div className="vote-count">
                                    <span className="count">{getVoteCount(playlist.id)}</span>
                                    <span className="vote-label">votes</span>
                                </div>
                                {selectedPlaylist === playlist.id && (
                                    <div className="your-vote">Your Vote</div>
                                )}
                            </div>
                        </div>
                        <div className="playlist-info">
                            <h3>{genre.toUpperCase()}</h3>
                            <div className="playlist-details">
                                <span>{playlist.songCount} songs</span>
                                <div className="vote-percentage-bar">
                                    <div 
                                        className="fill"
                                        style={{ 
                                            width: `${(getVoteCount(playlist.id) / Math.max(1, users.length)) * 100}%`
                                        }}
                                    />
                                </div>
                            </div>
                        </div>
                    </div>
                ))}
            </div>
        </div>
    );

    const minPlayersToStart = 1;
    const canStartGame = isHost && users.length >= minPlayersToStart;

    return (
        <div className="waiting-room-page">
            <div className="background-overlay">
                <div className="rotating-gradient"></div>
            </div>
            
            <div className="waiting-room-header">
                <button className="back-button" onClick={handleLeaveRoom}>
                    <FontAwesomeIcon icon={faArrowLeft} />
                    <span>Zurück zur Startseite</span>
                </button>
                <h1>Warteraum</h1>
                <div className="room-code-container">
                    <div className="room-code" onClick={copyRoomCode}>
                        Code: <span className="code">{roomCode}</span> 
                        <FontAwesomeIcon icon={copied ? faCheck : faCopy} className={copied ? "copied" : ""} />
                    </div>
                    <div className="copy-hint">{copied ? "Kopiert!" : "Klicken zum Kopieren"}</div>
                </div>
            </div>
            
            <div className="waiting-room-layout">
                {renderPlaylistSection()}
                
                <div className="bottom-section">
                    <div className="users-panel">
                        <div className="panel-header">
                            <FontAwesomeIcon icon={faUsers} />
                            <h2>Spieler ({users.length})</h2>
                        </div>
                        <div className="users-list">
                            {users.length === 0 ? (
                                <div className="no-users">Keine Spieler im Raum</div>
                            ) : (
                                users.map((user) => (
                                    <div key={user.id} className={`user-item ${user.creator ? 'host' : ''}`}>
                                        {user.name} {user.creator && <span className="host-badge">Host</span>}
                                    </div>
                                ))
                            )}
                        </div>
                        {isHost && (
                            <div className="game-controls">
                                <button 
                                    className={`start-game-button ${canStartGame ? '' : 'disabled'}`}
                                    onClick={handleStartGame}
                                    disabled={!canStartGame}
                                >
                                    <FontAwesomeIcon icon={faPlayCircle} />
                                    Spiel starten
                                </button>
                                {!canStartGame && users.length < minPlayersToStart && (
                                    <div className="start-hint">Mindestens {minPlayersToStart} Spieler benötigt</div>
                                )}
                            </div>
                        )}
                    </div>

                    <div className="chat-panel">
                        <div className="panel-header">
                            <FontAwesomeIcon icon={faMessage} />
                            <h2>Chat</h2>
                        </div>
                        <div className="chat-messages">
                            {messages.map((message, index) => (
                                <div key={index} className={`message ${message.system ? 'system-message' : ''}`}>
                                    {message.system ? (
                                        <span className="message-text system">{message.text}</span>
                                    ) : (
                                        <>
                                            <span className="message-sender">{message.sender}:</span>
                                            <span className="message-text">{message.text}</span>
                                        </>
                                    )}
                                </div>
                            ))}
                            <div ref={messageEndRef}></div>
                        </div>
                        <div className="chat-input">
                            <input
                                type="text"
                                value={inputValue}
                                onChange={(e) => setInputValue(e.target.value)}
                                onKeyPress={(e) => e.key === 'Enter' && handleSendMessage()}
                                placeholder="Gib eine Nachricht ein..."
                            />
                            <button onClick={handleSendMessage}>Senden</button>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    );
};
