let cachedSongs = null;
let fetchPromise = null;


export const fetchPlaylistSongs = async (socketInstance) => {
  if (cachedSongs && cachedSongs.length > 0) {
    console.log("Returning cached songs:", cachedSongs.length);
    return cachedSongs;
  }
  
  if (fetchPromise) {
    console.log("Request already in progress, waiting for response...");
    return fetchPromise;
  }
  
  console.log("Starting new fetch request for playlist songs");
  
  fetchPromise = new Promise(async (resolve, reject) => {
    try {
      let socket = socketInstance;
      
      if (!socket) {
        try {
          const socketModule = await import('@/common/contexts/SocketContext');
          socket = socketModule.socket;
        } catch (error) {
          console.error("Failed to import socket:", error);
        }
      }
      
      if (!socket) {
        throw new Error("Socket not available");
      }
      
      if (!socket.connected) {
        console.warn("Socket not connected - attempting to connect");
        socket.connect();

        await new Promise(resolve => setTimeout(resolve, 500));
        
        if (!socket.connected) {
          throw new Error("Could not connect to server");
        }
      }
      
      console.log("Socket connected, requesting songs...");

      const responsePromise = new Promise((resolve, reject) => {
        const handleResponse = (data) => {
          console.log("Received playlist-songs response");
          
          if (data.error) {
            console.error("Server returned error:", data.error);
            reject(new Error(data.error));
            return;
          }
          
          if (data && data.songs && Array.isArray(data.songs) && data.songs.length > 0) {
            cachedSongs = data.songs;
            resolve(data.songs);
          } else {
            console.error("Invalid song data in response", data);
            reject(new Error("Invalid song data"));
          }
        };

        socket.once("playlist-songs", handleResponse);

        setTimeout(() => {
          socket.off("playlist-songs", handleResponse);
          reject(new Error("Request timed out"));
        }, 10000);
      });

      socket.emit("get-playlist-songs");

      const songs = await responsePromise;
      fetchPromise = null;
      resolve(songs);
      
    } catch (error) {
      console.error("Error fetching playlist songs:", error);
      fetchPromise = null;
      reject(error);
    }
  });
  
  return fetchPromise.catch(error => {
    console.error("Failed to fetch playlist songs:", error);

    return [{
      id: 1,
      title: "Connection Error",
      artist: "Please refresh the page",
      coverUrl: "https://place-hold.it/500x500/f00/fff?text=Error",
      youtubeId: "dQw4w9WgXcQ"
    }];
  });
};

/**
 * Gets a single song by ID
 */
export const getSongById = async (id, socketInstance) => {
  try {
    const songs = await fetchPlaylistSongs(socketInstance);
    return songs.find(song => song.id === id) || null;
  } catch (error) {
    console.error("Error getting song by ID:", error);
    return null;
  }
};
